==> gtick-0.2.2.tar.gz: gtick-0.2.2/src/gtick.c <==
/*
 * GTick  -  A GTK+ Metronome
 *
 * gtick.c: Main app code
 * 
 *
 * Copyright (c) 1999, Alex Roberts
 * Copyright (c) 2003, Roland Stigge <stigge@antcom.de>
 *
 * GTick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GTick is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GTick; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/* GNU headers */
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

/* GTK+ headers */
#include <gtk/gtk.h>

/* own headers */
#include "../config.h"
#include "globals.h"
#include "metro.h"
#include "dsp.h"
#include "mixer.h"
#include "about.h"

/* icons */
#include "icon32x32.xpm"
#include "icon48x48.xpm"
#include "icon64x64.xpm"

metro_t* metro;
int debug;

void gtQuit(GtkWidget* w _U_, GtkWidget* window _U_)
{
  /* gtk_widget_destroy (window); */
  gtk_main_quit();
}

void gtAdjVol(GtkWidget* widget _U_, gpointer data)
{
  metro_t *metro = (metro_t *)data;
  gint vol;
  
  vol = -(-100 - GTK_ADJUSTMENT(metro->vAdj)->value);
  
  gtSetMixer (vol);
}

/*
 * Rate callback
 */
void gtUpdateRate (GtkWidget* widget _U_, gpointer data)
{
  metro_t *metro = (metro_t *)data;
  
  metro->period =
    1000/((GTK_ADJUSTMENT(metro->adjustment)->value)/60.0);

  if (debug)
    g_print ("gtUpdateRate: rate=%f running at %d ms\n",
	     (GTK_ADJUSTMENT(metro->adjustment)->value), metro->period);
}

/*
 * Callback for metronome on/off toggler
 */
void gtOnOff(GtkWidget* widget _U_, gpointer data)
{
metro_t *metro = (metro_t *)data;
  
  if (!metro->ticking)
    {
      /* Initialise /dev/dsp */ 
      if ((gtInitDSP(metro)) == -1) {
	fprintf(stderr, "Error on DSP init. Exit.");
        exit (1);
      }

      gtk_frame_set_label(GTK_FRAME(metro->box3), _("Metronome (On)"));
      gtk_label_set_text(GTK_LABEL(metro->togglebtn_label),
			 _("Stop Metronome"));
      metro->ticking = TRUE;
    }
  else
    {
      gtCloseDSP(metro);

      gtk_frame_set_label (GTK_FRAME(metro->box3), _("Metronome (Off)")); 
      gtk_label_set_text(GTK_LABEL(metro->togglebtn_label),
			 _("Start Metronome"));
      metro->ticking = FALSE;
    }
}

/*
 * Callback for meter mode
 */
void gtMeter(GtkWidget *widget, gpointer data) {
  metro_t *metro = (metro_t*) data;

  if (widget == metro->btn0 ||
      widget == metro->btn24 ||
      widget == metro->btn34 ||
      widget == metro->btn44)
    gtk_widget_set_sensitive(metro->meterspinbutton, FALSE);
  
  if (widget == metro->btn0) 
    metro->meter = 1;
  else if (widget == metro->btn24)
    metro->meter = 2;
  else if (widget == metro->btn34)
    metro->meter = 3;
  else if (widget == metro->btn44)
    metro->meter = 4;
  else { // spinbutton activated or changed
    if (widget == metro->btnother) // other: spinbutton activated
      gtk_widget_set_sensitive(metro->meterspinbutton, TRUE);
    metro->meter = gtk_spin_button_get_value_as_int(
	GTK_SPIN_BUTTON(metro->meterspinbutton));
  }

  if (debug)
    g_print("new meter: %d\n", metro->meter);
}

void gtInit(metro_t *metro)
{
  GtkObject* adjustment;
  GList* icon_list = NULL;
  
  metro->ticking = FALSE; /* set default */
  
  metro->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  gtk_signal_connect(GTK_OBJECT(metro->window), "destroy",
		     GTK_SIGNAL_FUNC (gtQuit),
		     &metro->window);

  gtk_window_set_title (GTK_WINDOW (metro->window), TITLE " " VERSION);
  gtk_container_set_border_width (GTK_CONTAINER (metro->window), 5);

  icon_list = g_list_append(icon_list,
          (gpointer) gdk_pixbuf_new_from_xpm_data((const char**) icon32x32));
  icon_list = g_list_append(icon_list,
          (gpointer) gdk_pixbuf_new_from_xpm_data((const char**) icon48x48));
  icon_list = g_list_append(icon_list,
          (gpointer) gdk_pixbuf_new_from_xpm_data((const char**) icon64x64));
  gtk_window_set_icon_list(GTK_WINDOW(metro->window), icon_list);

  metro->box1 = gtk_vbox_new (FALSE, 5);
  gtk_container_add(GTK_CONTAINER(metro->window), metro->box1);
  gtk_widget_show (metro->box1);

/*-- Metronome UI Code*/
  metro->box3 = gtk_frame_new(_("Metronome (Off)"));
  gtk_container_add (GTK_CONTAINER (metro->box1), metro->box3);
  gtk_widget_show (metro->box3);

  metro->box4 = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width (GTK_CONTAINER (metro->box4), 10);
  gtk_container_add (GTK_CONTAINER (metro->box3), metro->box4);
  gtk_widget_show (metro->box4);
  
  metro->box2 = gtk_hbox_new(FALSE, 10);
  gtk_container_add (GTK_CONTAINER (metro->box4), metro->box2);
  gtk_widget_show (metro->box2);

  metro->label = gtk_label_new(_("Speed:"));
  gtk_box_pack_start (GTK_BOX (metro->box2), metro->label, TRUE, TRUE, 0);
  gtk_widget_show (metro->label);

  metro->adjustment = gtk_adjustment_new (75.0, 30.0, 251.0, 1.0, 1.0, 1.0);
  gtk_signal_connect (GTK_OBJECT (metro->adjustment), "value_changed",
				  GTK_SIGNAL_FUNC (gtUpdateRate),
				  (metro_t *)metro);
		      
  gtUpdateRate (metro->window, (metro_t *)metro); /* Set initial rate */

  metro->scale = gtk_hscale_new (GTK_ADJUSTMENT (metro->adjustment));
  gtk_widget_set_size_request (GTK_WIDGET (metro->scale), 150, 30);
  gtk_range_set_update_policy (GTK_RANGE (metro->scale), GTK_UPDATE_DELAYED);
  gtk_scale_set_digits (GTK_SCALE (metro->scale), 1);
  gtk_scale_set_draw_value (GTK_SCALE (metro->scale), TRUE);
  gtk_box_pack_start (GTK_BOX (metro->box2), metro->scale, TRUE, TRUE, 0);
  gtk_widget_show (metro->scale);

  
  metro->box2 = gtk_hbox_new (FALSE, 10);
  gtk_container_add (GTK_CONTAINER (metro->box4), metro->box2);
  gtk_widget_show (metro->box2);

  metro->label = gtk_label_new (_("Volume:"));
  gtk_box_pack_start (GTK_BOX (metro->box2), metro->label, TRUE, TRUE, 0);
  gtk_widget_show (metro->label);
	  
  metro->vAdj = gtk_adjustment_new (-50, -100.0, 0.0, 5.0, 25.0, 0.0);
  gtk_signal_connect (GTK_OBJECT (metro->vAdj), "value_changed",
				  GTK_SIGNAL_FUNC (gtAdjVol),
				  (metro_t *)metro);	
  gtk_adjustment_set_value(GTK_ADJUSTMENT(metro->vAdj), 
			   (gfloat)(-100 + gtGetMixer()));
  metro->vol = gtk_hscale_new (GTK_ADJUSTMENT(metro->vAdj));
  gtk_widget_set_size_request (GTK_WIDGET (metro->vol), 150, 30);
  gtk_range_set_update_policy (GTK_RANGE (metro->vol), GTK_UPDATE_CONTINUOUS);
  gtk_scale_set_digits (GTK_SCALE (metro->vol), 0);
  gtk_scale_set_draw_value (GTK_SCALE (metro->vol), FALSE);
  gtk_box_pack_start (GTK_BOX (metro->box2), metro->vol, TRUE, TRUE, 0);
  gtk_widget_show (metro->vol);
  
/*-----------*/

/*-- Timing UI Code*/
  metro->meter = 1;		/* Default
					   *
					   * 1 = None
					   * 2 = 2/4
					   * 3 = 3/4
					   * 4 = 4/4
					   */

  metro->box4 = gtk_frame_new (_("Beat"));
  gtk_container_add (GTK_CONTAINER (metro->box1), metro->box4);
  gtk_widget_show (metro->box4);

  metro->box2 = gtk_hbox_new (FALSE, 20);
  gtk_container_set_border_width (GTK_CONTAINER (metro->box2), 10);
  gtk_container_add (GTK_CONTAINER (metro->box4), metro->box2);
  gtk_widget_show (metro->box2);

  metro->btn0 = gtk_radio_button_new_with_label (NULL, _("Even"));
  gtk_box_pack_start (GTK_BOX (metro->box2), metro->btn0, TRUE, TRUE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (metro->btn0), TRUE);
  gtk_widget_show (metro->btn0);

  gtk_signal_connect(GTK_OBJECT(metro->btn0), "pressed",
		     GTK_SIGNAL_FUNC(gtMeter),
		     metro);
  
  metro->btn24 =
    gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(metro->btn0),
	"2/4");
  gtk_box_pack_start(GTK_BOX(metro->box2), metro->btn24, TRUE, TRUE, 0);
  gtk_widget_show (metro->btn24);
  gtk_signal_connect(GTK_OBJECT(metro->btn24), "pressed",
		     GTK_SIGNAL_FUNC(gtMeter),
		     metro);
  
  metro->btn34 =
    gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(metro->btn24),
	"3/4");
  gtk_box_pack_start(GTK_BOX(metro->box2), metro->btn34, TRUE, TRUE, 0);
  gtk_widget_show (metro->btn34);
  gtk_signal_connect(GTK_OBJECT(metro->btn34), "pressed",
		     GTK_SIGNAL_FUNC(gtMeter),
		     metro);

  metro->btn44 =
    gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(metro->btn34),
	"4/4");
  gtk_box_pack_start(GTK_BOX(metro->box2), metro->btn44, TRUE, TRUE, 0);
  gtk_widget_show (metro->btn44);
  gtk_signal_connect(GTK_OBJECT(metro->btn44), "pressed",
		     GTK_SIGNAL_FUNC(gtMeter),
		     metro);

  metro->btnother =
    gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(metro->btn44),
	_("Other:"));
  gtk_box_pack_start(GTK_BOX(metro->box2), metro->btnother,
      TRUE, TRUE, 0);
  gtk_widget_show(metro->btnother);
  gtk_signal_connect(GTK_OBJECT(metro->btnother), "pressed",
      GTK_SIGNAL_FUNC(gtMeter), metro);
  
  adjustment = gtk_adjustment_new(5, 1, 99, 1, 4, 0);
  metro->meterspinbutton =
    gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 1, 0);
  gtk_widget_set_sensitive(metro->meterspinbutton, FALSE);
  gtk_widget_show(metro->meterspinbutton);
  gtk_box_pack_start(GTK_BOX(metro->box2), metro->meterspinbutton,
      TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(metro->meterspinbutton), "value-changed",
      GTK_SIGNAL_FUNC(gtMeter), metro);
/*-----------*/

/*-- General UI Code*/
  metro->separator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (metro->box1), metro->separator, FALSE,TRUE,0);
  gtk_widget_show (metro->separator);

  metro->box2 = gtk_hbox_new (FALSE, 10);
  gtk_container_set_border_width (GTK_CONTAINER (metro->box2), 5);
  gtk_box_pack_start (GTK_BOX (metro->box1), metro->box2, FALSE, TRUE, 0);
  gtk_widget_show (metro->box2);

  metro->togglebtn = gtk_button_new();
  gtk_signal_connect (GTK_OBJECT (metro->togglebtn), "clicked",
				  GTK_SIGNAL_FUNC (gtOnOff),
				  metro);
			   
  gtk_box_pack_start (GTK_BOX (metro->box2), metro->togglebtn, TRUE,TRUE,0);
  GTK_WIDGET_SET_FLAGS (metro->togglebtn, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (metro->togglebtn);
  gtk_widget_show (metro->togglebtn);

  metro->togglebtn_label = gtk_label_new(_("Start Metronome"));
  gtk_container_add(GTK_CONTAINER(metro->togglebtn),
		    metro->togglebtn_label);
  gtk_widget_show(metro->togglebtn_label);

  metro->button = gtk_button_new_with_label (_("About"));
  g_signal_connect_swapped(GTK_OBJECT(metro->button), "clicked",
		           G_CALLBACK(gtAboutBox),
		           metro);
			   
  gtk_box_pack_start (GTK_BOX (metro->box2), metro->button, TRUE, TRUE, 0);
  gtk_widget_show(metro->button);

  metro->button = gtk_button_new_with_label (_("Quit"));
  gtk_signal_connect_object(GTK_OBJECT(metro->button), "clicked",
			    GTK_SIGNAL_FUNC (gtQuit),
			    GTK_OBJECT(metro->window));
					   
  gtk_box_pack_start (GTK_BOX (metro->box2), metro->button, TRUE,TRUE,0);
  gtk_widget_show (metro->button);

  gtk_widget_show (metro->window);

/*-----------*/
}

int main(int argc, char *argv[])
{
  struct option long_options[] = {
    {"help",     no_argument,       0, 'h'},
    {"usage",    no_argument,       0, 'h'},
    {"version",  no_argument,       0, 'v'},
    {"debug",    optional_argument, 0, 'd'},
    {0, 0, 0, 0}
  };
  char *short_options = "hvd::";
  int option_index = 0;
  int c;

  /* prepare for i18n */
#ifdef ENABLE_NLS
  setlocale(LC_ALL, "");
  if (!bindtextdomain(PACKAGE, LOCALEDIR)) {
    fprintf(stderr, "Error setting directory for textdomain (i18n).\n");
  }
  if (!bind_textdomain_codeset(PACKAGE, "UTF-8")) {
    fprintf(stderr, "Error setting gettext output codeset to UTF-8.\n");
  }
  if (!textdomain(PACKAGE)) {
    fprintf(stderr, "Error setting domainname for gettext() "
                    "(internationalization).\n");
  }
#endif

  /* Initialise GTK+ */
  gtk_init (&argc, &argv);

  while ((c = getopt_long(argc, argv, short_options, long_options,
			  &option_index)) != -1) {
    switch(c) {
    case 'h': /* help */
      printf(_("\
Usage: %s [OPTION...]\n\
\n\
Options:\n\
  -h, --help              Show this help message\n\
  -v, --version           Print version information\n\
  -d, --debug[=level]     Print additional runtime debugging data to stdout\n\
\n"),
      argv[0]);
      exit(0);
    case 'v': /* version */
      printf(PACKAGE " " VERSION "\n");
      exit(0);
    case 'd': /* debug mode */
      if (optarg) {
	debug = strtol(optarg, NULL, 0);
      } else {
	debug = 1;
      }
      break;
    case '?':
      exit(1);
    }
  }
  /* no further arguments expected, so not handled */

  /* Initialise /dev/mixer */
  gtInitMixer();

  metro = g_malloc(sizeof(metro_t));
  
  /* Initialise UI */
  gtInit(metro);
  
  /* GTK+ internal main loop */
  gtk_main();

  if (metro->ticking)
    gtCloseDSP(metro);

  /* Shut down /dev/dsp and /dev/mixer and end program */
  gtCloseMixer();
  
  g_free(metro);
  return 0;
}

==> gtick-0.2.3.tar.gz: gtick-0.2.3/src/gtick.c <==
/*
 * GTick  -  A GTK+ Metronome
 *
 * gtick.c: Main app code
 * 
 *
 * Copyright (c) 1999, Alex Roberts
 * Copyright (c) 2003, Roland Stigge <stigge@antcom.de>
 *
 * GTick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GTick is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GTick; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <config.h>

/* GNU headers */
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

/* GTK+ headers */
#include <gtk/gtk.h>

/* own headers */
#include "globals.h"
#include "metro.h"
#include "dsp.h"
#include "mixer.h"
#include "util.h"

int debug;

metro_t* metro;

int main(int argc, char *argv[])
{
  struct option long_options[] = {
    {"help",     no_argument,       0, 'h'},
    {"usage",    no_argument,       0, 'h'},
    {"version",  no_argument,       0, 'v'},
    {"debug",    optional_argument, 0, 'd'},
    {0, 0, 0, 0}
  };
  char *short_options = "hvd::";
  int option_index = 0;
  int c;

  /* prepare for i18n */
#ifdef ENABLE_NLS
  setlocale(LC_ALL, "");
  if (!bindtextdomain(PACKAGE, LOCALEDIR)) {
    fprintf(stderr, "Error setting directory for textdomain (i18n).\n");
  }
  if (!textdomain(PACKAGE)) {
    fprintf(stderr, "Error setting domainname for gettext() "
                    "(internationalization).\n");
  }
#endif

  /* Initialise GTK+ */
  gtk_init (&argc, &argv);

  while ((c = getopt_long(argc, argv, short_options, long_options,
			  &option_index)) != -1) {
    switch(c) {
    case 'h': /* help */
      printf(_("\
Usage: %s [OPTION...]\n\
\n\
Options:\n\
  -h, --help              Show this help message\n\
  -v, --version           Print version information\n\
  -d, --debug[=level]     Print additional runtime debugging data to stdout\n\
\n"),
      argv[0]);
      exit(0);
    case 'v': /* version */
      printf(PACKAGE " " VERSION "\n");
      exit(0);
    case 'd': /* debug mode */
      if (optarg) {
	debug = strtol(optarg, NULL, 0);
      } else {
	debug = 1;
      }
      break;
    case '?':
      exit(1);
    }
  }
  /* no further arguments expected, so not handled */

#ifdef ENABLE_NLS
  if (!bind_textdomain_codeset(PACKAGE, "UTF-8")) { /* needed for GTK */
    fprintf(stderr, "Error setting gettext output codeset to UTF-8.\n");
  }
#endif
  
  /* Initialise /dev/mixer */
  gtInitMixer();

  /* Initialise UI */
  metro = metro_new();
  
  /* GTK+ internal main loop */
  gtk_main();

  if (metro->state == STATE_RUNNING)
    gtCloseDSP(metro);

  /* Shut down /dev/dsp and /dev/mixer and end program */
  gtCloseMixer();
  
  metro_delete(metro);
  return 0;
}
